@@ -1,11 +1,20 @@
+#' @include sql-build.R
+
 #' @export
 #' @rdname sql_build
-join_query <- function(x, y, vars, type = "inner", by = NULL, suffix = c(".x", ".y"), na_matches = FALSE) {
+join_query <- function(x,
+                       y,
+                       select,
+                       ...,
+                       type = "inner",
+                       by = NULL,
+                       suffix = c(".x", ".y"),
+                       na_matches = FALSE) {
   structure(
     list(
       x = x,
       y = y,
-      vars = vars,
+      select = select,
       type = type,
       by = by,
       na_matches = na_matches
@@ -14,13 +23,13 @@ join_query <- function(x, y, vars, type = "inner", by = NULL, suffix = c(".x", "
   )
 }
 
-multi_join_query <- function(x, joins, table_vars, vars) {
+multi_join_query <- function(x, joins, table_names, select) {
   structure(
     list(
       x = x,
       joins = joins,
-      table_vars = table_vars,
-      vars = vars
+      table_names = table_names,
+      select = select
     ),
     class = c("multi_join_query", "query")
   )
@@ -34,10 +43,10 @@ print.join_query <- function(x, ...) {
   cat_line(indent(paste0(x$by$x, "-", x$by$y)))
 
   cat_line("X:")
-  cat_line(indent_print(sql_build(x$x)))
+  cat_line(indent_print(x$x))
 
   cat_line("Y:")
-  cat_line(indent_print(sql_build(x$y)))
+  cat_line(indent_print(x$y))
 }
 
 #' @export
@@ -45,7 +54,7 @@ print.multi_join_query <- function(x, ...) {
   cat_line("<SQL JOINS>")
 
   cat_line("X:")
-  cat_line(indent_print(sql_build(x$x)))
+  cat_line(indent_print(x$x))
 
   for (i in vctrs::vec_seq_along(x$joins)) {
     cat_line("Type: ", paste0(x$joins$type[[i]]))
@@ -54,12 +63,17 @@ print.multi_join_query <- function(x, ...) {
     cat_line(indent(paste0(x$joins$by[[i]]$x, "-", x$joins$by[[i]]$y)))
 
     cat_line("Y:")
-    cat_line(indent_print(sql_build(x$joins$table[[i]])))
+    cat_line(indent_print(x$joins$table[[i]]))
   }
 }
 
 #' @export
-sql_render.join_query <- function(query, con = NULL, ..., subquery = FALSE, lvl = 0) {
+sql_render.join_query <- function(query,
+                                  con = NULL,
+                                  ...,
+                                  sql_options = NULL,
+                                  subquery = FALSE,
+                                  lvl = 0) {
   from_x <- sql_render(query$x, con, ..., subquery = TRUE, lvl = lvl + 1)
   from_y <- sql_render(query$y, con, ..., subquery = TRUE, lvl = lvl + 1)
 
@@ -68,29 +82,60 @@ sql_render.join_query <- function(query, con = NULL, ..., subquery = FALSE, lvl
     type = query$type,
     by = query$by,
     na_matches = query$na_matches,
+    select = query$select,
     lvl = lvl
   )
 }
 
 #' @export
-sql_render.multi_join_query <- function(query, con = NULL, ..., subquery = FALSE, lvl = 0) {
+sql_render.multi_join_query <- function(query,
+                                        con = NULL,
+                                        ...,
+                                        sql_options = NULL,
+                                        subquery = FALSE,
+                                        lvl = 0) {
   x <- sql_render(query$x, con, ..., subquery = TRUE, lvl = lvl + 1)
   query$joins$table <- purrr::map(
     query$joins$table,
-    ~ sql_render(.x, con, ..., subquery = TRUE, lvl = lvl + 1)
+    function(table) sql_render(table, con, ..., subquery = TRUE, lvl = lvl + 1)
   )
 
   sql_query_multi_join(
     con = con,
     x = x,
     joins = query$joins,
-    table_vars = query$table_vars,
+    table_names = query$table_names,
     by_list = query$by_list,
-    vars = query$vars,
+    select = query$select,
     lvl = lvl
   )
 }
 
+#' @export
+flatten_query.join_query <- flatten_query_2_tables
+
+#' @export
+flatten_query.multi_join_query <- function(qry, query_list) {
+  x <- qry$x
+  query_list_new <- flatten_query(x, query_list)
+  qry$x <- get_subquery_name(x, query_list_new)
+
+  for (i in vctrs::vec_seq_along(qry$joins)) {
+    y <- qry$joins$table[[i]]
+    query_list_new <- flatten_query(y, query_list_new)
+    qry$joins$table[[i]] <- get_subquery_name(y, query_list_new)
+  }
+
+  # TODO reuse query
+  name <- unique_subquery_name()
+  wrapped_query <- set_names(list(qry), name)
+
+  query_list$queries <- c(query_list_new$queries, wrapped_query)
+  query_list$name <- name
+  query_list
+}
+
+
 # SQL generation ----------------------------------------------------------
 
 #' @param vars tibble with three columns:
@@ -128,65 +173,33 @@ sql_render.multi_join_query <- function(query, con = NULL, ..., subquery = FALSE
 #'    list(c("x", "a"), c("x", "a"))
 #' )
 #'
-#' # Full join
-#' vars <- tibble(
-#'   name = c("y", "a", "b", "c"),
-#'   table = list(c(1, 2), 1, 1, 2),
-#'   var = list(c("y", "y"), "a", "b", "c")
-#' )
-#' sql_multi_join_vars(
-#'    simulate_dbi(),
-#'    vars,
-#'    c("one", "two"),
-#'    list(c("x", "a"), c("x", "a"))
-#' )
-sql_multi_join_vars <- function(con, vars, table_vars) {
+#' # Full and right join are handled via `sql_rf_join_vars`
+sql_multi_join_vars <- function(con, vars, table_vars, use_star, qualify_all_columns) {
   all_vars <- tolower(unlist(table_vars))
-  duplicated_vars <- all_vars[vctrs::vec_duplicate_detect(all_vars)]
-  duplicated_vars <- unique(duplicated_vars)
+  if (qualify_all_columns) {
+    duplicated_vars <- unique(all_vars)
+  } else {
+    duplicated_vars <- all_vars[vctrs::vec_duplicate_detect(all_vars)]
+    duplicated_vars <- unique(duplicated_vars)
+  }
   table_names <- names(table_vars)
 
   # FIXME vectorise `sql_table_prefix()` (need to update `ident()` and friends for this...)
-  ns_table <- vctrs::list_sizes(vars$table)
-  ns_var <- vctrs::list_sizes(vars$var)
-
-  if (any(ns_table != ns_var) || any(ns_table == 0) || any(ns_var == 0)) {
-    cli_abort("vars$var or vars$table has incorrect size", .internal = TRUE)
-  }
-
-  # Handle `full_join()` which must COALESCE join cols and can never use *
-  if (any(ns_table > 1)) {
-    out <- purrr::map2(
-      vars$table, vars$var,
-      function(table_ids, vars) {
-        sql_multi_join_var(con, vars, table_ids, table_names, duplicated_vars)
-      }
-    )
-
-    out <- set_names(out, vars$name)
-    return(sql(unlist(out)))
-  }
-
-  # At this point `vars$var` and `vars$table` must be list of scalars so they
-  # can be safely unchopped
   out <- rep_named(vars$name, list())
-  vars$var <- vctrs::list_unchop(vars$var, ptype = character())
-  vars$table <- vctrs::list_unchop(vars$table, ptype = integer())
 
   for (i in seq_along(table_names)) {
-    all_vars_current <- table_vars[[i]]
-    vars_idx <- which(vars$table == i)
-    used_vars_current <- vars$var[vars_idx]
-    out_vars_current <- vars$name[vars_idx]
+    all_vars_i <- table_vars[[i]]
+    vars_idx_i <- which(vars$table == i)
+    used_vars_i <- vars$var[vars_idx_i]
+    out_vars_i <- vars$name[vars_idx_i]
 
-    if (join_can_use_star(all_vars_current, used_vars_current, out_vars_current, vars_idx)) {
-      id <- vars_idx[[1]]
-      tbl_alias <- escape(ident(table_names[i]), con = con)
-      out[[id]] <- sql(paste0(tbl_alias, ".*"))
+    if (use_star && join_can_use_star(all_vars_i, used_vars_i, out_vars_i, vars_idx_i)) {
+      id <- vars_idx_i[[1]]
+      out[[id]] <- sql_star(con, table_names[i])
       names(out)[id] <- ""
     } else {
-      out[vars_idx] <- purrr::map2(
-        used_vars_current, i,
+      out[vars_idx_i] <- purrr::map2(
+        used_vars_i, i,
         ~ sql_multi_join_var(con, .x, .y, table_names, duplicated_vars)
       )
 
@@ -218,66 +231,89 @@ join_can_use_star <- function(all_vars, used_vars, out_vars, idx) {
 
 sql_multi_join_var <- function(con, var, table_id, table_names, duplicated_vars) {
   if (length(table_id) > 1) {
-    if (length(table_id) != 2 || length(var) != 2) {
-      cli_abort("{.arg table_id} or {.arg var} does not have length 2", .internal = TRUE)
-    }
-
-    table_1 <- table_names[[table_id[[1]]]]
-    table_2 <- table_names[[table_id[[2]]]]
-
-    out <- sql_expr(
-      COALESCE(
-        !!sql_table_prefix(con, var[[1]], table = ident(table_1)),
-        !!sql_table_prefix(con, var[[2]], table = ident(table_2))
-      ),
-      con = con
-    )
-
-    return(out)
+    # TODO use `vec_check_size()` after vctrs 0.6 release
+    cli_abort("{.arg table_id} must have size 1", .internal = TRUE)
   }
 
   if (tolower(var) %in% duplicated_vars) {
-    sql_table_prefix(con, var, ident(table_names[[table_id]]))
+    sql_table_prefix(con, var, table_names[[table_id]])
   } else {
     sql_escape_ident(con, var)
   }
 }
 
-sql_join_vars <- function(con, vars, x_as = "LHS", y_as = "RHS", type) {
+sql_rf_join_vars <- function(con,
+                             type,
+                             vars,
+                             x_as = "LHS",
+                             y_as = "RHS",
+                             use_star,
+                             qualify_all_columns) {
+  type <- arg_match0(type, c("right", "full"))
+  table_names <- c(unclass(x_as), unclass(y_as))
+
+  if (type == "full") {
+    duplicated_vars <- intersect(tolower(vars$all_x), tolower(vars$all_y))
+    out <- purrr::map2(
+      vars$x, vars$y,
+      ~ {
+        if (!is.na(.x) && !is.na(.y)) {
+          out <- sql_expr(
+            COALESCE(
+              !!sql_table_prefix(con, .x, table = x_as),
+              !!sql_table_prefix(con, .y, table = y_as)
+            ),
+            con = con
+          )
+
+          return(out)
+        }
+
+        if (!is.na(.x)) {
+          sql_multi_join_var(con, .x, 1L, table_names, duplicated_vars)
+        } else {
+          sql_multi_join_var(con, .y, 2L, table_names, duplicated_vars)
+        }
+      }
+    )
+
+    out <- set_names(out, vars$name)
+    return(sql(unlist(out)))
+  }
+
   multi_join_vars <- purrr::map2_dfr(
     vars$x, vars$y,
     ~ {
-      table <- integer()
-      var <- character()
       if (!is.na(.x)) {
         table <- 1L
         var <- .x
+      } else {
+        table <- 2L
+        var <- .y
       }
 
-      if (!is.na(.y)) {
-        table <- c(table, 2L)
-        var <- c(var, .y)
-      }
-
-      vctrs::new_data_frame(list(table = list(table), var = list(var)), n = 1L)
+      vctrs::new_data_frame(list(table = table, var = var), n = 1L)
     }
   )
-  multi_join_vars <- vctrs::vec_cbind(name = vars$alias, multi_join_vars)
-  table_vars <- set_names(vars[c("all_x", "all_y")], c(unclass(x_as), unclass(y_as)))
+
+  multi_join_vars <- vctrs::vec_cbind(name = vars$name, multi_join_vars)
+  table_vars <- set_names(vars[c("all_x", "all_y")], table_names)
 
   sql_multi_join_vars(
-    con,
-    multi_join_vars,
-    table_vars = table_vars
+    con = con,
+    vars = multi_join_vars,
+    table_vars = table_vars,
+    use_star = use_star,
+    qualify_all_columns = qualify_all_columns
   )
 }
 
-sql_join_tbls <- function(con, by, na_matches = "never") {
+sql_join_tbls <- function(con, by, na_matches) {
   na_matches <- arg_match(na_matches, c("na", "never"))
 
   if (na_matches == "na" || length(by$x) + length(by$y) > 0) {
-    lhs <- sql_table_prefix(con, by$x, by$x_as %||% ident("LHS"))
-    rhs <- sql_table_prefix(con, by$y, by$y_as %||% ident("RHS"))
+    lhs <- sql_table_prefix(con, by$x, by$x_as %||% "LHS")
+    rhs <- sql_table_prefix(con, by$y, by$y_as %||% "RHS")
 
     if (na_matches == "na") {
       compare <- purrr::map_chr(seq_along(lhs), function(i) {
@@ -295,9 +331,24 @@ sql_join_tbls <- function(con, by, na_matches = "never") {
 }
 
 sql_table_prefix <- function(con, var, table = NULL) {
+  if (!is_bare_character(var)) {
+    cli_abort("{.arg var} must be a bare character.", .internal = TRUE)
+  }
   var <- sql_escape_ident(con, var)
 
   if (!is.null(table)) {
+    table <- as_table_ident(table)
+    table <- escape(table, collapse = NULL, con = con)
+    sql(paste0(table, ".", var))
+  } else {
+    var
+  }
+}
+
+sql_star <- function(con, table = NULL) {
+  var <- sql("*")
+  if (!is.null(table)) {
+    table <- as_table_ident(table)
     table <- escape(table, collapse = NULL, con = con)
     sql(paste0(table, ".", var))
   } else {
