@@ -95,7 +95,7 @@ rows_insert.tbl_lazy <- function(x,
                                  method = NULL) {
   check_dots_empty()
   rows_check_in_place(x, in_place)
-  name <- target_table_name(x, in_place)
+  table <- target_table(x, in_place)
 
   conflict <- rows_check_conflict(conflict)
 
@@ -111,11 +111,12 @@ rows_insert.tbl_lazy <- function(x,
 
   returning_cols <- rows_check_returning(x, returning, enexpr(returning))
 
-  if (!is_null(name)) {
+  if (!is_null(table)) {
     sql <- sql_query_insert(
       con = remote_con(x),
-      x_name = name,
-      y = y,
+      table = table,
+      from = sql_render(y, remote_con(x), lvl = 1),
+      insert_cols = colnames(y),
       by = by,
       ...,
       conflict = conflict,
@@ -153,18 +154,19 @@ rows_append.tbl_lazy <- function(x,
                                  returning = NULL) {
   check_dots_empty()
   rows_check_in_place(x, in_place)
-  name <- target_table_name(x, in_place)
+  table <- target_table(x, in_place)
 
   rows_check_containment(x, y)
   y <- rows_auto_copy(x, y, copy = copy)
 
   returning_cols <- rows_check_returning(x, returning, enexpr(returning))
 
-  if (!is_null(name)) {
+  if (!is_null(table)) {
     sql <- sql_query_append(
       con = remote_con(x),
-      x_name = name,
-      y = y,
+      table = table,
+      from = sql_render(y, remote_con(x), lvl = 1),
+      insert_cols = colnames(y),
       ...,
       returning_cols = returning_cols
     )
@@ -200,7 +202,7 @@ rows_update.tbl_lazy <- function(x,
                                  returning = NULL) {
   check_dots_empty()
   rows_check_in_place(x, in_place)
-  name <- target_table_name(x, in_place)
+  table <- target_table(x, in_place)
 
   rows_check_containment(x, y)
   y <- rows_auto_copy(x, y, copy = copy)
@@ -217,7 +219,7 @@ rows_update.tbl_lazy <- function(x,
   returning_cols <- rows_check_returning(x, returning, enexpr(returning))
 
 
-  if (!is_null(name)) {
+  if (!is_null(table)) {
     # TODO handle `returning_cols` here
     if (is_empty(new_columns)) {
       return(invisible(x))
@@ -226,14 +228,14 @@ rows_update.tbl_lazy <- function(x,
     con <- remote_con(x)
     update_cols <- setdiff(colnames(y), by)
     update_values <- set_names(
-      sql_table_prefix(con, update_cols, ident("...y")),
+      sql_table_prefix(con, update_cols, "...y"),
       update_cols
     )
 
     sql <- sql_query_update_from(
       con = con,
-      x_name = name,
-      y = y,
+      table = table,
+      from = sql_render(y, remote_con(y), lvl = 1),
       by = by,
       update_values = update_values,
       ...,
@@ -271,16 +273,16 @@ rows_update.tbl_lazy <- function(x,
 #' @importFrom dplyr rows_patch
 #' @rdname rows-db
 rows_patch.tbl_lazy <- function(x,
-                                 y,
-                                 by = NULL,
-                                 ...,
-                                 unmatched = c("error", "ignore"),
-                                 copy = FALSE,
-                                 in_place = FALSE,
-                                 returning = NULL) {
+                                y,
+                                by = NULL,
+                                ...,
+                                unmatched = c("error", "ignore"),
+                                copy = FALSE,
+                                in_place = FALSE,
+                                returning = NULL) {
   check_dots_empty()
   rows_check_in_place(x, in_place)
-  name <- target_table_name(x, in_place)
+  table <- target_table(x, in_place)
 
   rows_check_containment(x, y)
   y <- rows_auto_copy(x, y, copy = copy)
@@ -296,7 +298,7 @@ rows_patch.tbl_lazy <- function(x,
 
   returning_cols <- rows_check_returning(x, returning, enexpr(returning))
 
-  if (!is_null(name)) {
+  if (!is_null(table)) {
     # TODO handle `returning_cols` here
     if (is_empty(new_columns)) {
       return(invisible(x))
@@ -306,15 +308,15 @@ rows_patch.tbl_lazy <- function(x,
 
     update_cols <- setdiff(colnames(y), by)
     update_values <- sql_coalesce(
-      sql_table_prefix(con, update_cols, name),
-      sql_table_prefix(con, update_cols, ident("...y"))
+      sql_table_prefix(con, update_cols, table),
+      sql_table_prefix(con, update_cols, "...y")
     )
     update_values <- set_names(update_values, update_cols)
 
     sql <- sql_query_update_from(
       con = con,
-      x_name = name,
-      y = y,
+      table = table,
+      from = sql_render(y, remote_con(y), lvl = 1),
       by = by,
       update_values = update_values,
       ...,
@@ -369,7 +371,7 @@ rows_upsert.tbl_lazy <- function(x,
                                  method = NULL) {
   check_dots_empty()
   rows_check_in_place(x, in_place)
-  name <- target_table_name(x, in_place)
+  table <- target_table(x, in_place)
 
   rows_check_containment(x, y)
   y <- rows_auto_copy(x, y, copy = copy)
@@ -383,7 +385,7 @@ rows_upsert.tbl_lazy <- function(x,
 
   new_columns <- setdiff(colnames(y), by)
 
-  if (!is_null(name)) {
+  if (!is_null(table)) {
     # TODO use `rows_insert()` here
     if (is_empty(new_columns)) {
       return(invisible(x))
@@ -391,8 +393,8 @@ rows_upsert.tbl_lazy <- function(x,
 
     sql <- sql_query_upsert(
       con = remote_con(x),
-      x_name = name,
-      y = y,
+      table = table,
+      from = sql_render(y, remote_con(x), lvl = 1),
       by = by,
       update_cols = setdiff(colnames(y), by),
       ...,
@@ -444,7 +446,7 @@ rows_delete.tbl_lazy <- function(x,
                                  returning = NULL) {
   check_dots_empty()
   rows_check_in_place(x, in_place)
-  name <- target_table_name(x, in_place)
+  table <- target_table(x, in_place)
 
   rows_check_containment(x, y)
   y <- rows_auto_copy(x, y, copy = copy)
@@ -464,11 +466,11 @@ rows_delete.tbl_lazy <- function(x,
     inform(message, class = c("dplyr_message_delete_extra_cols", "dplyr_message"))
   }
 
-  if (!is_null(name)) {
+  if (!is_null(table)) {
     sql <- sql_query_delete(
       con = remote_con(x),
-      x_name = name,
-      y = y,
+      table = table,
+      from = sql_render(y, remote_con(x), lvl = 2),
       by = by,
       ...,
       returning_cols = returning_cols
@@ -559,11 +561,6 @@ has_returned_rows <- function(x) {
   !identical(attr(x, "returned_rows"), NULL)
 }
 
-#' @export
-sql_returning_cols.duckdb_connection <- function(con, cols, ...) {
-  cli_abort("DuckDB does not support the {.arg returning} argument.")
-}
-
 sql_coalesce <- function(x, y) {
   sql(paste0("COALESCE(", x, ", ", y, ")"))
 }
@@ -707,53 +704,51 @@ tick <- function(x) {
 
 # other helpers -----------------------------------------------------------
 
-target_table_name <- function(x, in_place) {
+target_table <- function(x, in_place) {
   # Never touch target table with `in_place = FALSE`
   if (!is_true(in_place)) {
     return(NULL)
   }
 
-  name <- remote_name(x)
-  if (is_null(name)) {
+  table <- remote_table(x)
+  if (is_null(table)) {
     cli_abort("Can't determine name for target table. Set {.code in_place = FALSE} to return a lazy table.")
   }
 
-  name
+  table
 }
 
-rows_prep <- function(con, x_name, y, by, lvl = 0) {
-  y_name <- ident("...y")
-  from <- dbplyr_sql_subquery(con,
-    sql_render(y, con, subquery = TRUE, lvl = lvl + 1),
-    name = y_name,
-    lvl = lvl
-  )
-
-  join_by <- list(x = by, y = by, x_as = y_name, y_as = x_name, condition = "=")
+rows_prep <- function(con, table, from, by, lvl = 0) {
+  y_name <- "...y"
+  join_by <- list(x = by, y = by, x_as = y_name, y_as = table, condition = "=")
   where <- sql_join_tbls(con, by = join_by, na_matches = "never")
 
   list(
-    from = from,
+    from = sql_query_wrap(con, from, y_name, lvl = lvl),
     where = where
   )
 }
 
-rows_insert_prep <- function(con, x_name, y, by, lvl = 0) {
-  out <- rows_prep(con, x_name, y, by, lvl = lvl)
+rows_insert_prep <- function(con, table, from, cols, by, lvl = 0) {
+  out <- rows_prep(con, table, from, by, lvl = lvl)
 
-  join_by <- list(x = by, y = by, x_as = x_name, y_as = ident("...y"), condition = "=")
+  join_by <- list(x = by, y = by, x_as = table, y_as = "...y", condition = "=")
   where <- sql_join_tbls(con, by = join_by, na_matches = "never")
-  out$conflict_clauses <- sql_clause_where_exists(x_name, where, not = TRUE)
+  out$conflict_clauses <- sql_clause_where_exists(table, where, not = TRUE)
 
-  insert_cols <- escape(ident(colnames(y)), collapse = ", ", parens = TRUE, con = con)
-  out$insert_clause <- sql_clause_insert(con, insert_cols, x_name)
+  insert_cols <- escape(ident(cols), collapse = ", ", parens = TRUE, con = con)
+  out$insert_clause <- sql_clause_insert(con, insert_cols, table)
 
   out
 }
 
 rows_auto_copy <- function(x, y, copy, call = caller_env()) {
-  name <- remote_name(x)
-  x_types <- get_col_types(remote_con(x), name, call)
+  if (same_src(x, y)) {
+    return(y)
+  }
+
+  table <- remote_table(x)
+  x_types <- db_col_types(remote_con(x), table, call)
 
   if (!is_null(x_types)) {
     rows_check_containment(x, y, error_call = call)
@@ -763,34 +758,6 @@ rows_auto_copy <- function(x, y, copy, call = caller_env()) {
   auto_copy(x, y, copy = copy, types = x_types)
 }
 
-get_col_types <- function(con, name, call) {
-  if (is_null(name)) {
-    return(NULL)
-  }
-
-  UseMethod("get_col_types")
-}
-
-#' @export
-get_col_types.TestConnection <- function(con, name, call) {
-  NULL
-}
-
-#' @export
-get_col_types.DBIConnection <- function(con, name, call) {
-  NULL
-}
-
-#' @export
-get_col_types.PqConnection <- function(con, name, call) {
-  name <- as.sql(name, con)
-  res <- DBI::dbSendQuery(con, paste0("SELECT * FROM ", name))
-  on.exit(DBI::dbClearResult(res))
-  DBI::dbFetch(res, n = 0)
-  col_info_df <- DBI::dbColumnInfo(res)
-  set_names(col_info_df[[".typname"]], col_info_df[["name"]])
-}
-
 rows_get_or_execute <- function(x, sql, returning_cols, call = caller_env()) {
   con <- remote_con(x)
   msg <- "Can't modify database table {.val {remote_name(x)}}."
