@@ -36,6 +36,7 @@
 #'   and `%in%`.
 #' @param multiple,unmatched Unsupported in database backends. As a workaround
 #'   for multiple use a unique key and for unmatched a foreign key constraint.
+#' @param relationship Unsupported in database backends.
 #' @param x_as,y_as Alias to use for `x` resp. `y`. Defaults to `"LHS"` resp.
 #'   `"RHS"`
 #' @inherit arrange.tbl_lazy return
@@ -79,6 +80,7 @@ inner_join.tbl_lazy <- function(x,
                                 na_matches = c("never", "na"),
                                 multiple = NULL,
                                 unmatched = "drop",
+                                relationship = NULL,
                                 sql_on = NULL,
                                 auto_index = FALSE,
                                 x_as = NULL,
@@ -95,6 +97,7 @@ inner_join.tbl_lazy <- function(x,
     na_matches = na_matches,
     multiple = multiple,
     unmatched = unmatched,
+    relationship = relationship,
     sql_on = sql_on,
     auto_index = auto_index,
     x_as = x_as,
@@ -117,6 +120,7 @@ left_join.tbl_lazy <- function(x,
                                na_matches = c("never", "na"),
                                multiple = NULL,
                                unmatched = "drop",
+                               relationship = NULL,
                                sql_on = NULL,
                                auto_index = FALSE,
                                x_as = NULL,
@@ -133,6 +137,7 @@ left_join.tbl_lazy <- function(x,
     na_matches = na_matches,
     multiple = multiple,
     unmatched = unmatched,
+    relationship = relationship,
     sql_on = sql_on,
     auto_index = auto_index,
     x_as = x_as,
@@ -155,6 +160,7 @@ right_join.tbl_lazy <- function(x,
                                 na_matches = c("never", "na"),
                                 multiple = NULL,
                                 unmatched = "drop",
+                                relationship = NULL,
                                 sql_on = NULL,
                                 auto_index = FALSE,
                                 x_as = NULL,
@@ -171,6 +177,7 @@ right_join.tbl_lazy <- function(x,
     na_matches = na_matches,
     multiple = multiple,
     unmatched = unmatched,
+    relationship = relationship,
     sql_on = sql_on,
     auto_index = auto_index,
     x_as = x_as,
@@ -192,6 +199,7 @@ full_join.tbl_lazy <- function(x,
                                keep = NULL,
                                na_matches = c("never", "na"),
                                multiple = NULL,
+                               relationship = NULL,
                                sql_on = NULL,
                                auto_index = FALSE,
                                x_as = NULL,
@@ -207,6 +215,7 @@ full_join.tbl_lazy <- function(x,
     keep = keep,
     na_matches = na_matches,
     multiple = multiple,
+    relationship = relationship,
     sql_on = sql_on,
     auto_index = auto_index,
     x_as = x_as,
@@ -314,6 +323,7 @@ add_join <- function(x,
                      na_matches = "never",
                      multiple = NULL,
                      unmatched = "drop",
+                     relationship = NULL,
                      sql_on = NULL,
                      auto_index = FALSE,
                      x_as = NULL,
@@ -321,33 +331,27 @@ add_join <- function(x,
                      call = caller_env()) {
   x_names <- tbl_vars(x)
   y_names <- tbl_vars(y)
-  if (identical(by, character()) && is.null(sql_on)) {
-    if (type != "cross") {
-      lifecycle::deprecate_soft(
-        when = "1.1.0",
-        what = I("Using `by = character()` to perform a cross join"),
-        with = "cross_join()",
-        env = caller_env(),
-        user_env = caller_env(2)
-      )
-    }
-    type <- "cross"
-  }
 
-  if (!is.null(sql_on)) {
-    by <- list(x = character(0), y = character(0), on = unclass(sql_on))
-  } else if (type == "cross") {
-    by <- list(x = character(0), y = character(0))
-  } else if (is_null(by)) {
-    by <- join_by_common(x_names, y_names, error_call = call)
-  } else {
-    by <- dbplyr_as_join_by(by, error_call = call)
-  }
+  by <- join_prepare_by(
+    by = by,
+    sql_on = sql_on,
+    type = type,
+    x_names = x_names,
+    y_names = y_names,
+    error_call = call
+  )
+  type <- by$type
 
   check_join_by_supported(by, call = call)
 
   check_join_multiple(multiple, by, call = call)
   check_join_unmatched(unmatched, by, call = call)
+  check_unsupported_arg(
+    relationship,
+    allowed = "many-to-many",
+    allow_null = TRUE,
+    call = call
+  )
 
   y <- auto_copy(
     x, y,
@@ -370,7 +374,6 @@ add_join <- function(x,
   # the table alias can only be determined after `select()` was inlined.
   # This works even though `by` is used in `join_inline_select()` and updated
   # because this does not touch `by$x_as` and `by$y_as`.
-  # TODO can this be really be done before inlining?
   join_alias <- make_join_aliases(x_as, y_as, sql_on, call)
 
   inline_result <- join_inline_select(x$lazy_query, by$x, by$on)
@@ -378,39 +381,50 @@ add_join <- function(x,
   x_vars <- inline_result$vars
   by$x <- inline_result$by
 
-  new_query <- join_needs_new_query(x$lazy_query, join_alias, type)
-  if (new_query) {
-    x_join_vars <- tibble(
-      name = op_vars(x),
-      table = list(1L),
-      var = as.list(x_vars)
-    )
-    table_id <- 2L
-  } else {
-    x_join_vars <- x_lq$vars
-    table_id <- vctrs::vec_size(x_lq$table_names) + 1L
-  }
-
   inline_result <- join_inline_select(y$lazy_query, by$y, by$on)
   y_lq <- inline_result$lq
   y_vars <- inline_result$vars
   by$y <- inline_result$by
 
-  y_join_vars <- tibble(
-    name = op_vars(y),
-    table = list(table_id),
-    var = as.list(y_vars)
-  )
+  if (type %in% c("full", "right")) {
+    vars <- rf_join_vars(
+      x_vars = x_vars,
+      y_vars = y_vars,
+      vars_info = vars,
+      keep = keep,
+      condition = by$condition,
+      by_y = by$y,
+      type = type
+    )
 
+    table_names_x <- make_table_names(join_alias$x, x_lq)
+    table_names_y <- make_table_names(join_alias$y, y_lq)
+    out <- lazy_rf_join_query(
+      x = x_lq,
+      y = y_lq,
+      type = type,
+      by = list(
+        x = by$x,
+        y = by$y,
+        condition = by$condition,
+        on = sql(by$on),
+        na_matches = na_matches
+      ),
+      table_names = vctrs::vec_rbind(table_names_x, table_names_y),
+      vars = vars
+    )
+
+    return(out)
+  }
+
+  new_query <- join_needs_new_query(x$lazy_query, join_alias, type)
   vars <- multi_join_vars(
-    x_join_vars = x_join_vars,
-    y_join_vars = y_join_vars,
+    x_lq = x_lq,
+    x_vars = x_vars,
+    y_vars = y_vars,
+    new_query = new_query,
     vars_info = vars,
-    keep = keep,
-    condition = by$condition,
-    by_y = by$y,
-    type = type,
-    table_id = table_id,
+    error_call = call
   )
 
   joins_data <- new_joins_data(
@@ -476,10 +490,6 @@ join_needs_new_query <- function(x_lq, join_alias, type) {
     return(TRUE)
   }
 
-  if (type %in% c("full", "right")) {
-    return(TRUE)
-  }
-
   x_as <- join_alias$x
   y_as <- join_alias$y
 
@@ -504,58 +514,110 @@ join_needs_new_query <- function(x_lq, join_alias, type) {
   FALSE
 }
 
-multi_join_vars <- function(x_join_vars,
-                            y_join_vars,
+multi_join_vars <- function(x_lq,
+                            x_vars,
+                            y_vars,
+                            new_query,
                             vars_info,
-                            keep,
-                            condition,
-                            by_y,
-                            type,
-                            table_id,
-                            call) {
-  x_join_vars <- vctrs::vec_slice(x_join_vars, vars_info$x$out)
-  x_join_vars$name <- names(vars_info$x$out)
-  y_join_vars <- vctrs::vec_slice(y_join_vars, vars_info$y$out)
-  y_join_vars$name <- names(vars_info$y$out)
+                            error_call) {
+  if (new_query) {
+    x_join_vars <- tibble(
+      name = names(vars_info$x$out),
+      table = 1L,
+      var = vctrs::vec_slice(x_vars, vars_info$x$out)
+    )
+    table_id <- 2L
+  } else {
+    x_join_vars <- vctrs::vec_slice(x_lq$vars, vars_info$x$out)
+    x_join_vars$name <- names(vars_info$x$out)
+    table_id <- vctrs::vec_size(x_lq$table_names) + 1L
+  }
+
+  y_join_vars <- tibble(
+    name = names(vars_info$y$out),
+    table = table_id,
+    var = vctrs::vec_slice(y_vars, vars_info$y$out)
+  )
+
+  vctrs::vec_rbind(x_join_vars, y_join_vars, .error_call = error_call)
+}
+
+rf_join_vars <- function(x_vars,
+                         y_vars,
+                         vars_info,
+                         keep,
+                         condition,
+                         by_y,
+                         type,
+                         call) {
+  x_join_vars <- tibble(
+    name = names(vars_info$x$out),
+    x = vctrs::vec_slice(x_vars, vars_info$x$out),
+    y = NA_character_
+  )
+
+  y_join_vars <- tibble(
+    name = names(vars_info$y$out),
+    x = NA_character_,
+    y = vctrs::vec_slice(y_vars, vars_info$y$out)
+  )
 
   keep <- keep %||% (condition != "==")
   idx <- vars_info$x$key[!keep]
-  if (type %in% c("left", "inner")) {
-    # use all variables from `x` as is
-    # use non-join variables from `y`
-  } else if (type == "right") {
-    # Careful: this relies on the assumption that right_join()` starts a new query
+  if (type == "right") {
     # `x`: non-join variables; `y`: all variables
-    # -> must update table id of `x` join vars
-    x_join_vars$table[idx] <- list(table_id)
-    x_join_vars$var[idx] <- as.list(by_y[!keep])
-  } else if (type == "full") {
-    # Careful: this relies on the assumption that `full_join()` starts a new query
-    x_join_vars$table[idx] <- purrr::map(
-      x_join_vars$table[idx],
-      ~ c(.x, table_id)
-    )
-    x_join_vars$var[idx] <- purrr::map2(
-      x_join_vars$var[idx], by_y[!keep],
-      ~ c(.x, .y)
-    )
-  } else if (type == "cross") {
-    # -> simply append `y_rows`
+    x_join_vars$x[idx] <- NA
   }
+  x_join_vars$y[idx] <- by_y[!keep]
 
   vctrs::vec_rbind(x_join_vars, y_join_vars)
 }
 
+
+join_prepare_by <- function(by,
+                            sql_on,
+                            type,
+                            x_names,
+                            y_names,
+                            error_call,
+                            env = caller_env(2),
+                            user_env = caller_env(3)) {
+  if (identical(by, character()) && is.null(sql_on)) {
+    if (type != "cross") {
+      lifecycle::deprecate_soft(
+        when = "1.1.0",
+        what = I("Using `by = character()` to perform a cross join"),
+        with = "cross_join()",
+        env = env,
+        user_env = user_env
+      )
+    }
+    type <- "cross"
+  }
+
+  if (!is.null(sql_on)) {
+    by <- list(x = character(0), y = character(0), on = unclass(sql_on))
+  } else if (type == "cross") {
+    by <- list(x = character(0), y = character(0))
+  } else if (is_null(by)) {
+    by <- join_by_common(x_names, y_names, error_call = error_call)
+  } else {
+    by <- dbplyr_as_join_by(by, error_call = error_call)
+  }
+
+  by$type <- type
+  by
+}
+
 new_joins_data <- function(x_lq, y_lq, new_query, type, by, na_matches) {
   if (new_query) {
     by_x_table_id <- rep_along(by$x, 1L)
   } else {
     idx <- vctrs::vec_match(by$x, x_lq$vars$name)
-    vctrs::list_check_all_size(x_lq$vars$var[idx], size = 1)
 
     # need to fix `by$x` in case it was renamed in an inlined select
-    by$x <- unlist(x_lq$vars$var)[idx]
-    by_x_table_id <- unlist(x_lq$vars$table)[idx]
+    by$x <- x_lq$vars$var[idx]
+    by_x_table_id <- x_lq$vars$table[idx]
   }
 
   tibble(
@@ -563,8 +625,8 @@ new_joins_data <- function(x_lq, y_lq, new_query, type, by, na_matches) {
     type = type,
     by_x_table_id = list(by_x_table_id),
     by = list(list(
-      x = ident(by$x),
-      y = ident(by$y),
+      x = by$x,
+      y = by$y,
       condition = by$condition,
       on = sql(by$on),
       na_matches = na_matches
@@ -595,6 +657,7 @@ add_semi_join <- function(x,
   }
 
   check_join_by_supported(by, call = call)
+  na_matches <- arg_match(na_matches, c("na", "never"), error_call = call)
 
   y <- auto_copy(
     x, y, copy,
@@ -603,35 +666,77 @@ add_semi_join <- function(x,
 
   inline_result <- join_inline_select(x$lazy_query, by$x, sql_on)
   x_lq <- inline_result$lq
+  x_vars <- inline_result$vars
   by$x <- inline_result$by
+
+  inline_result <- semi_join_inline_select(y$lazy_query, by$y, sql_on)
+  y_lq <- inline_result$lq
+  by$y <- inline_result$by
+  where <- inline_result$where
+
   vars <- tibble(
     name = op_vars(x),
-    var = inline_result$vars
+    var = x_vars
   )
+  by$na_matches <- na_matches
 
   # the table alias can only be determined after `select()` was inlined
   join_alias <- make_join_aliases(x_as, y_as, sql_on, call)
 
   x_alias <- make_table_names(join_alias$x, x_lq)
-  y_alias <- make_table_names(join_alias$y, y)
+  y_alias <- make_table_names(join_alias$y, y_lq)
   by[c("x_as", "y_as")] <- join_two_table_alias(
     c(x_alias$name, y_alias$name),
     c(x_alias$from, y_alias$from)
   )
-  by$x_as <- ident(by$x_as)
-  by$y_as <- ident(by$y_as)
 
   lazy_semi_join_query(
     x_lq,
-    y$lazy_query,
+    y_lq,
     vars = vars,
     anti = anti,
     by = by,
-    na_matches = na_matches,
+    where = where,
     call = call
   )
 }
 
+can_inline_semi_join <- function(x) {
+  is_lazy_select_query_simple(
+    x,
+    ignore_where = TRUE,
+    ignore_group_by = TRUE,
+    select = "projection"
+  )
+}
+
+semi_join_inline_select <- function(lq, by, on) {
+  if (is_empty(on) && can_inline_semi_join(lq)) {
+    vars <- purrr::map_chr(lq$select$expr, as_string)
+
+    idx <- vctrs::vec_match(lq$select$name, by)
+    by <- vctrs::vec_assign(by, idx, vars)
+
+    lq_org <- lq
+    lq <- lq$x
+    lq$group_vars <- op_grps(lq_org)
+    lq$order_vars <- op_sort(lq_org)
+    lq$frame <- op_frame(lq_org)
+
+    where <- lq_org$where
+  } else {
+    vars <- op_vars(lq)
+    where <- NULL
+  }
+
+  list(
+    lq = lq,
+    vars = vars,
+    by = by,
+    where = where
+  )
+}
+
 make_join_aliases <- function(x_as, y_as, sql_on, call) {
   x_as <- check_join_as1(x_as, arg = "x_as", sql_on, default = "LHS", call)
   y_as <- check_join_as1(y_as, arg = "y_as", sql_on, default = "RHS", call)
@@ -644,14 +749,14 @@ make_join_aliases <- function(x_as, y_as, sql_on, call) {
 }
 
 make_table_names <- function(as, lq) {
-  name <- unclass(query_name(lq))
+  name <- unclass(remote_name(lq, null_if_local = FALSE))
 
   if (!is.null(as)) {
     tibble(name = as, from = "as")
   } else if (!is.null(name)) {
     tibble(name = name, from = "name")
   } else {
-    tibble(name = NA_character_, from = "")
+    tibble(name = "", from = "")
   }
 }
 
@@ -705,27 +810,26 @@ join_two_table_alias <- function(names, from) {
   check_character(from)
   vctrs::vec_assert(names, size = 2L)
 
-  out <- dplyr::coalesce(names, c("LHS", "RHS"))
+  out <- names
+  out[from == ""] <- c("LHS", "RHS")[from == ""]
 
   if (!identical(out[1], out[2])) {
     return(out)
   }
   # -> must rename
 
-  if (from[1] != "as" && from[2] != "as") {
-    # self join of named table
-    if (!is.na(names[1]) && !is.na(names[2]) && identical(names[1], names[2])) {
-      out <- c(
-        paste0(names[1], "_LHS"),
-        paste0(names[2], "_RHS")
-      )
-      return(out)
-    }
+  tables_have_same_name <- from[1] == "name" && from[2] == "name" && identical(names[1], names[2])
+  if (tables_have_same_name) {
+    out <- c(
+      paste0(names[1], "_LHS"),
+      paste0(names[2], "_RHS")
+    )
+    return(out)
   }
 
   out_repaired <- vctrs::vec_as_names(out, repair = "unique", quiet = TRUE)
-  may_repair <- c(from[1] != "as", from[2] != "as")
-  out[may_repair] <- out_repaired[may_repair]
+  may_repair_name <- c(from[1] != "as", from[2] != "as")
+  out[may_repair_name] <- out_repaired[may_repair_name]
 
   out
 }
